//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// dumprom.H
//
//		Console Win32 Application which uses the 1394 Class API
//		from TI to dump a 1394 device ROM.
//
//		Developed using MSVC 5.0.
//
//	10/13/98 Bill Alexander
//		Created
//
//	Created with tabs set to every 3 spaces (i.e. 4, 7, ...)
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma pack(1)

#ifndef _1394API_H_
#include "1394api.h"
#endif

#ifdef _APP_MAIN_MODULE_
#define DUMPROM_EXTERN
#else
#define DUMPROM_EXTERN extern
#endif

// Default 1394 Device GUID and Number
#define DEFAULT_DEVICE_NUMBER				1
#define DEFAULT_DEVICE_ID					0xFFFFFFFF


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Global Variables
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// Verbosity Level Command Line Option
DUMPROM_EXTERN
int gVerbosityLevel;

// Stop before exit command line option
DUMPROM_EXTERN
BOOL gStopBeforeExit;

// Flag to indicate whether or not the 1394 class driver is loaded
// and initialized
DUMPROM_EXTERN
BOOL gDriverInitialized;

// Specified offset option
DUMPROM_EXTERN
BOOL gUseSpecifiedOffset;
DUMPROM_EXTERN
DWORD gSpecifiedOffset;

// Connection handle to 1394 device
DUMPROM_EXTERN
cls1394HANDLE ghDev;

// Device Mode -- Do we use command prompt Device ID, Device Number or list and let user chose?
DUMPROM_EXTERN
DWORD gDeviceMode;

// Device Id
DUMPROM_EXTERN
DWORD gDeviceId;

// Device Number
DUMPROM_EXTERN
DWORD gDeviceNumber;

// Configuration ROM Buffer and Size
DUMPROM_EXTERN
PQUADLET gCfgRomBuf;
DUMPROM_EXTERN
DWORD gCfgRomBufSize;

// MyPrintf VerbosityLevel settings
#define VL_NONE								0
#define VL_ALWAYS								0
#define VL_ERROR								0
#define VL_MAJOR_STEP						1
#define VL_DEBUG								2

// Device Modes
#define DM_USE_DEVICE_NUMBER				1
#define DM_USE_DEVICE_ID					2
#define DM_LIST_DEVICES_AND_CHOOSE		3
#define DM_DEFAULT							DM_USE_DEVICE_NUMBER
															
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// IEEE1212 and IEEE1394 Definitions
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

typedef struct _1394BusInfoBlockStruct			
{															
	// First quadlet
	QUADLET		bus_name;					// Always 0x31333934 or "1394"
															
	// Second quadlet									
	QUADLET		reserved1:					12;
	QUADLET		max_rec:						4;
	QUADLET		cyc_clk_acc:				8;
	QUADLET		reserved2:					4;
	QUADLET		bmc:							1;
	QUADLET		isc:							1;
	QUADLET		cmc:							1;
	QUADLET		irmc:							1;
													
	// Third quadlet							
	QUADLET		chip_id_hi:					8;
	QUADLET		node_vendor_id:			24;

	// Fourth quadlet
	QUADLET		chip_id_lo;

} _1394BusInfoBlockT, *P_1394BusInfoBlockT;

#define BUS_NAME_1394	0x31333934				
															
typedef struct CfgRomHeaderStruct
{
	union
	{
		// General 1394 CFG ROM format
		struct
		{
			QUADLET		rom_crc_value:		16;
			QUADLET		crc_length:			8;
			QUADLET		info_length:		8;		// Should be 0x04 (for 1394)
			_1394BusInfoBlockT	bus_info_block;
		}gcr;

		// Minimal CFG ROM format
		struct
		{
			QUADLET		vendor_id:			24;
			QUADLET		info_length:		8;		// Should be 0x01
		}mcr;
	};
} CfgRomHeaderT, *PCfgRomHeaderT;

typedef struct BlockHeaderStruct
{
	QUADLET	crc:								16;
	QUADLET	length:							16;
} BlockHeaderT, *PBlockHeaderT;

typedef struct DirEntryStruct
{
	union
	{
		// Immediate value (key_type = 00b)
		struct
		{
			QUADLET	value:					24;
			QUADLET	key_value:				6;
			QUADLET	key_type:				2;
		}iv;

		// Pointer to location relative to XXXXFFFF:F0000000 (key_type = 01b)
		struct
		{
			QUADLET	csr_offset:				24;
			QUADLET	key_value:				6;
			QUADLET	key_type:				2;
		}co;

		// Pointer to location relative to the position of this value (key_type = 10b & 11b)
		struct
		{
			QUADLET	indirect_offset:		24;
			QUADLET	key_value:				6;
			QUADLET	key_type:				2;
		}io;

		// General format
		struct
		{
			QUADLET	value:					24;
			QUADLET	key:						8;
		}general;

		// QUADLET format
		QUADLET		q;

	};
} DirEntryT, *PDirEntryT;

// Definitions for values of key_type
// NOTE:  1212 spec does not specify the exact name of these values
// so we'll name them
#define KT_IMMEDIATE_VALUE					0
#define KT_CSR_OFFSET						1
#define KT_LEAF_INDIRECT_OFFSET			2
#define KT_DIRECTORY_INDIRECT_OFFSET	3

// Definitions for values of key_value
// NOTE:  We will name these exactly as the 1212 spec defines
#define KV_UNKNOWN							0x00
#define Textual_Descriptor					0x01
#define Bus_Dependent_Info					0x02
#define Module_Vendor_Id					0x03
#define Module_Hw_Version					0x04
#define Module_Spec_Id						0x05
#define Module_Sw_Version					0x06
#define Module_Dependent_Info				0x07
#define Node_Vendor_Info					0x08
#define Node_Hw_Version						0x09
#define Node_Spec_Id							0x0A
#define Node_Sw_Version						0x0B
#define Node_Capabilities					0x0C
#define Node_Unique_Id						0x0D
#define Node_Units_Extent					0x0E
#define Node_Memory_Extent					0x0F
#define Node_Dependent_Info				0x10
#define Unit_Directory						0x11
#define Unit_Spec_Id							0x12
#define Unit_Sw_Version						0x13
#define Unit_Dependent_Info				0x14
#define Unit_Location						0x15
#define Unit_Poll_Mask						0x16
// 1394 specific key_value definitions
#define Unit_Power_Requirements			0x30
// SBP-2 specific key_value definitions
#define Model_Id								0x17
#define Command_Set_Spec_Id				0x38
#define Command_Set							0x39
#define Unit_Characteristics				0x3A
#define Command_Set_Revision				0x3B
#define Firmware_Revision					0x3C
#define Management_Agent					0x14	// 0x54 (key_type = KT_CSR_OFFSET)
#define Logical_Unit_Directory			0x14	// 0xD4 (key_type = KT_DIRECTORY_INDIRECT_OFFSET)
#define Logical_Unit_Number				0x14

// Misc SBP-2 Definitions
#define SBP2_Unit_Spec_Id					0x00609E
#define SBP2_Unit_Sw_Version				0x010483
#define SBP2_Command_Set_Spec_Id			0x00609E
#define SBP2_Command_Set					0x0104D8

typedef struct NodeCapabiltiesStruct
{
	QUADLET init:								1;
	QUADLET ded:								1;
	QUADLET off:								1;
	QUADLET atn:								1;
	QUADLET elo:								1;
	QUADLET Reserved1:						1;
	QUADLET drg:								1;
	QUADLET lst:								1;
	QUADLET fix:								1;
	QUADLET _64:								1;
	QUADLET prv:								1;
	QUADLET bas:								1;
	QUADLET ext:								1;
	QUADLET _int:								1;
	QUADLET ms:									1;
	QUADLET spt:								1;
	QUADLET Reserved2:						8;
} NodeCapabilitiesT, *PNodeCapabilitiesT;

typedef struct NodeUnitsExtentStruct
{
	QUADLET number:							16;
	QUADLET align:								6;
	QUADLET Reserved:							2;
} NodeUnitsExtentT, *PNodeUnitsExtentT;

typedef NodeUnitsExtentT NodeMemoryExtentT, *PNodeMemoryExtentT;

typedef struct UnitPollMaskStruct
{
	QUADLET poll_mask:						16;
	QUADLET Reserved:							8;
} UnitPollMaskT, *PUnitPollMaskT;

typedef struct LogicalUnitNumberStruct
{
	QUADLET lun:								16;
	QUADLET device_type:						5;
	QUADLET Reserved1:						1;
	QUADLET o:									1;
	QUADLET Reserved2:						1;
} LogicalUnitNumberT, *PLogicalUnitNumberT;

// 1394 Core 1394 Addresses
LARGE_INTEGER CSR_BASE					=	{0xF0000000, 0x0000FFFF};		// 0xFFFF:F0000000
LARGE_INTEGER STATE_CLEAR				=	{0xF0000000, 0x0000FFFF};		// 0xFFFF:F0000000
LARGE_INTEGER STATE_SET					=	{0xF0000004, 0x0000FFFF};		// 0xFFFF:F0000004
LARGE_INTEGER NODE_IDS					=	{0xF0000008, 0x0000FFFF};		// 0xFFFF:F0000008
LARGE_INTEGER RESET_START				=	{0xF000000C, 0x0000FFFF};		// 0xFFFF:F000000C
LARGE_INTEGER SPLIT_TIMEOUT_H			=	{0xF0000018, 0x0000FFFF};		// 0xFFFF:F0000018
LARGE_INTEGER SPLIT_TIMEOUT_L			=	{0xF000001C, 0x0000FFFF};		// 0xFFFF:F000001C
LARGE_INTEGER BUSY_TIMEOUT				=	{0xF0000210, 0x0000FFFF};		// 0xFFFF:F0000210
LARGE_INTEGER CONFIG_ROM				=	{0xF0000400, 0x0000FFFF};		// 0xFFFF:F0000400


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Function Prototypes
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int main(int argc, char *argv[]);
BOOL ProcessCommandLine (int argc, char *argv[]);
void Usage ();
void MyPrintf (int vl, char *fmt);
void MyPrintf1 (int vl, char *fmt, DWORD p1);
void MyPrintf2 (int vl, char *fmt, DWORD p1, DWORD p2);
void MyPrintf3 (int vl, char *fmt, DWORD p1, DWORD p2, DWORD p3);
void MyPrintf4 (int vl, char *fmt, DWORD p1, DWORD p2, DWORD p3, DWORD p4);
void MyPrintf5 (int vl, char *fmt, DWORD p1, DWORD p2, DWORD p3, DWORD p4, DWORD p5);
void MyPrintf6 (int vl, char *fmt, DWORD p1, DWORD p2, DWORD p3, DWORD p4, DWORD p5, DWORD p6);
BOOL WriteQuadlet (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			QUADLET qValue);
BOOL WriteQuadletBuffer (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			PQUADLET Buffer,
			DWORD BufferSize);
BOOL WriteVerifyQuadlet (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			QUADLET qValue,
			DWORD to);
BOOL ReadQuadlet(cls1394HANDLE hDev, LARGE_INTEGER liAddress, QUADLET *qValue);
BOOL HostOpen (cls1394HANDLE *hHost);
BOOL HostReadNodeId (cls1394HANDLE hHost, QUADLET *qValue);
BOOL DeviceOpen (DWORD DeviceMode, PDWORD DeviceNumber, PDWORD DeviceId, cls1394HANDLE *hDev);
BOOL AllocateConfigRomBuffer (cls1394HANDLE hDev, PQUADLET *CfgRomBuf, PDWORD CfgRomBufSize);
BOOL ReadConfigRomBuffer (cls1394HANDLE hDev, PQUADLET CfgRomBuffer, DWORD CfgRomSize);
BOOL DumpSpecificQuadlet (cls1394HANDLE hDev, DWORD SpecifiedOffset);
BOOL DumpConfigRom (PQUADLET CfgRomBuffer, DWORD CfgRomSize);
BOOL DecodeConfigRom (PQUADLET CfgRomBuf, DWORD CfgRomBufSize);
void DumpQuadletAsChars (QUADLET q);
char *KeyToString (PDirEntryT DirEntry, DWORD UnitSpecId, DWORD UnitSwVersion);
char *ImmediateValueToString (PDirEntryT DirEntry, DWORD UnitSpecId, DWORD UnitSwVersion);
LARGE_INTEGER LI_HOST_SWAP (LARGE_INTEGER li);
void HostSwapQuadlet (QUADLET qValue, PQUADLET dst);
BOOL LoadLibraryAndFixPointers (void);


#ifdef DUMPROM_EXTERN
#undef DUMPROM_EXTERN
#endif
